@@ -1325,10 +1325,12 @@ _cogl_texture_2d_sliced_set_region (CoglTexture    *tex,
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  ctx->texture_driver->pixel_format_to_gl (_cogl_bitmap_get_format (bmp),
-                                           NULL, /* internal format */
-                                           &gl_format,
-                                           &gl_type);
+  bmp = _cogl_texture_prepare_for_upload (bmp,
+                                          cogl_texture_get_format (tex),
+                                          NULL,
+                                          NULL,
+                                          &gl_format,
+                                          &gl_type);
 
   /* Send data to GL */
   _cogl_texture_2d_sliced_upload_subregion_to_gl (tex_2ds,
@@ -1339,6 +1341,8 @@ _cogl_texture_2d_sliced_set_region (CoglTexture    *tex,
                                                   gl_format,
                                                   gl_type);
 
+  cogl_object_unref (bmp);
+
   return TRUE;
 }
 
