@@ -45,6 +45,8 @@
 
 typedef struct _CoglRendererWgl
 {
+  GModule *gl_module;
+
   /* Function pointers for GLX specific extensions */
 #define COGL_WINSYS_FEATURE_BEGIN(a, b, c, d, e, f)
 
@@ -122,14 +124,35 @@ static const CoglFeatureData winsys_feature_data[] =
   };
 
 static CoglFuncPtr
-_cogl_winsys_get_proc_address (const char *name)
+_cogl_winsys_renderer_get_proc_address (CoglRenderer *renderer,
+                                        const char *name)
 {
-  return (CoglFuncPtr) wglGetProcAddress ((LPCSTR) name);
+  CoglRendererWgl *wgl_renderer = renderer->winsys;
+  void *proc = wglGetProcAddress ((LPCSTR) name);
+
+  /* The documentation for wglGetProcAddress implies that it only
+     returns pointers to extension functions so if it fails we'll try
+     resolving the symbol directly from the the GL library */
+  if (proc == NULL)
+    {
+      if (wgl_renderer->gl_module == NULL)
+        wgl_renderer->gl_module = g_module_open ("opengl32", 0);
+
+      if (wgl_renderer->gl_module)
+        g_module_symbol (wgl_renderer->gl_module, name, &proc);
+    }
+
+  return proc;
 }
 
 static void
 _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 {
+  CoglRendererWgl *wgl_renderer = renderer->winsys;
+
+  if (wgl_renderer->gl_module)
+    g_module_close (wgl_renderer->gl_module);
+
   g_slice_free (CoglRendererWgl, renderer->winsys);
 }
 
@@ -569,7 +592,7 @@ update_winsys_features (CoglContext *context, GError **error)
       COGL_NOTE (WINSYS, "  WGL Extensions: %s", wgl_extensions);
 
       for (i = 0; i < G_N_ELEMENTS (winsys_feature_data); i++)
-        if (_cogl_feature_check (_cogl_context_get_winsys (context),
+        if (_cogl_feature_check (context->display->renderer,
                                  "WGL", winsys_feature_data + i, 0, 0,
                                  COGL_DRIVER_GL,
                                  wgl_extensions,
@@ -831,9 +854,9 @@ _cogl_winsys_wgl_get_vtable (void)
     {
       memset (&vtable, 0, sizeof (vtable));
 
-      vtable.id = COGL_WINSYS_ID_EGL;
+      vtable.id = COGL_WINSYS_ID_WGL;
       vtable.name = "WGL";
-      vtable.get_proc_address = _cogl_winsys_get_proc_address;
+      vtable.renderer_get_proc_address = _cogl_winsys_renderer_get_proc_address;
       vtable.renderer_connect = _cogl_winsys_renderer_connect;
       vtable.renderer_disconnect = _cogl_winsys_renderer_disconnect;
       vtable.display_setup = _cogl_winsys_display_setup;
